@@ -83,7 +83,7 @@ module Share = struct
     { prims = StringMap.empty; strings = StringMap.empty; applies = IntMap.empty }
 
   type t =
-    { mutable count : int aux
+    { count : int aux
     ; mutable vars : J.ident aux
     ; alias_prims : bool
     ; alias_strings : bool
@@ -939,7 +939,7 @@ let _ =
     register_bin_prim "caml_string_equal" `Pure (fun cx cy _ ->
       bool (J.EBin (J.EqEq, cx, cy)))
      ]}
-   *)
+*)
 
 (****)
 (* when raising ocaml exception and [improved_stacktrace] is enabled,
