@@ -24,8 +24,8 @@ let rec constant_of_const : _ -> Code.constant =
   function
   | Const_base (Const_int i) -> Int (Int32.of_int_warning_on_overflow i)
   | Const_base (Const_char c) -> Int (Int32.of_int (Char.code c))
-  | ((Const_base (Const_string (s, _)))[@if ocaml_version < (4, 11, 0)])
-  | ((Const_base (Const_string (s, _, _)))[@if ocaml_version >= (4, 11, 0)]) ->
+  | ((Const_base (Const_string (s, _))) [@if ocaml_version < (4, 11, 0)])
+  | ((Const_base (Const_string (s, _, _))) [@if ocaml_version >= (4, 11, 0)]) ->
       String s
   | Const_base (Const_float s) -> Float (float_of_string s)
   | Const_base (Const_int32 i) -> Int i
@@ -35,12 +35,14 @@ let rec constant_of_const : _ -> Code.constant =
   | Const_float_array sl ->
       let l = List.map ~f:(fun f -> Code.Float (float_of_string f)) sl in
       Tuple (Obj.double_array_tag, Array.of_list l, Unknown)
-  | ((Const_pointer (i, _))[@if BUCKLESCRIPT]) -> Int (Int32.of_int_warning_on_overflow i)
-  | ((Const_block (tag, _, l))[@if BUCKLESCRIPT]) ->
+  | ((Const_pointer (i, _)) [@if BUCKLESCRIPT]) ->
+      Int (Int32.of_int_warning_on_overflow i)
+  | ((Const_block (tag, _, l)) [@if BUCKLESCRIPT]) ->
       let l = Array.of_list (List.map l ~f:constant_of_const) in
       Tuple (tag, l, Unknown)
-  | ((Const_pointer i)[@ifnot BUCKLESCRIPT]) -> Int (Int32.of_int_warning_on_overflow i)
-  | ((Const_block (tag, l))[@ifnot BUCKLESCRIPT]) ->
+  | ((Const_pointer i) [@ifnot BUCKLESCRIPT] [@if ocaml_version < (4, 12, 0)]) ->
+      Int (Int32.of_int_warning_on_overflow i)
+  | ((Const_block (tag, l)) [@ifnot BUCKLESCRIPT]) ->
       let l = Array.of_list (List.map l ~f:constant_of_const) in
       Tuple (tag, l, Unknown)
 
@@ -72,6 +74,8 @@ let rec find_loc_in_summary ident' = function
 
 (* Copied from ocaml/utils/tbl.ml *)
 module Tbl = struct
+  [@@@ocaml.warning "-unused-field"]
+
   open Poly
 
   type ('a, 'b) t =
@@ -219,6 +223,8 @@ end
 [@@if ocaml_version >= (4, 8, 0)]
 
 module Ident = struct
+  [@@@ocaml.warning "-unused-field"]
+
   (* Copied from ocaml/typing/ident.ml *)
   type 'a tbl' =
     | Empty
