@@ -93,6 +93,7 @@ end = struct
     ; paths : string list
     ; source : string option
     }
+  [@@ocaml.warning "-unused-field"]
 
   module String_table = Hashtbl.Make (String)
   module Int_table = Hashtbl.Make (Int)
@@ -417,7 +418,7 @@ end = struct
       else if tag = Obj.double_tag
       then Float (Obj.magic x : float)
       else if tag = Obj.double_array_tag
-      then Float_array (Obj.magic x : float array)
+      then Float_array (Array.init (Obj.size x) ~f:(fun i -> Obj.double_field x i))
       else if tag = Obj.custom_tag
       then
         match ident_of_custom x with
@@ -749,10 +750,11 @@ let compiled_blocks = ref Addr.Map.empty
 let method_cache_id = ref 1
 
 let new_closure_repr =
-  (* true starting with 4.12 *)
   match Ocaml_version.v with
-  | `V4_02 | `V4_03 | `V4_04 | `V4_06 | `V4_07 | `V4_08 | `V4_09 | `V4_10 | `V4_11 ->
-      false
+  | `V4_04 | `V4_06 | `V4_07 | `V4_08 | `V4_09 | `V4_10 | `V4_11 -> false
+  | `V4_12 | `V4_13 -> true
+
+let clo_offset_3 = if new_closure_repr then 3 else 2
 
 type compile_info =
   { blocks : Blocks.u
@@ -1077,7 +1079,7 @@ and compile infos pc state instrs =
                 | GRAB -> getu code (addr + 1) + 1
                 | _ -> 1
               in
-              let offset = if new_closure_repr then i * 3 else i * 2 in
+              let offset = i * clo_offset_3 in
               let state' = State.start_function state env offset in
               let params, state' = State.make_stack nparams state' in
               if debug_parser () then Format.printf ") {@.";
@@ -1090,21 +1092,22 @@ and compile infos pc state instrs =
               Let (x, Closure (List.rev params, (addr, args))) :: instr)
         in
         compile infos (pc + 3 + nfuncs) (State.acc (nfuncs - 1) state) instrs
-    | OFFSETCLOSUREM2 -> compile infos (pc + 1) (State.env_acc (-2) state) instrs
+    | OFFSETCLOSUREM3 ->
+        compile infos (pc + 1) (State.env_acc (-clo_offset_3) state) instrs
     | OFFSETCLOSURE0 -> compile infos (pc + 1) (State.env_acc 0 state) instrs
-    | OFFSETCLOSURE2 -> compile infos (pc + 1) (State.env_acc 2 state) instrs
+    | OFFSETCLOSURE3 -> compile infos (pc + 1) (State.env_acc clo_offset_3 state) instrs
     | OFFSETCLOSURE ->
         let n = gets code (pc + 1) in
         compile infos (pc + 2) (State.env_acc n state) instrs
-    | PUSHOFFSETCLOSUREM2 ->
+    | PUSHOFFSETCLOSUREM3 ->
         let state = State.push state in
-        compile infos (pc + 1) (State.env_acc (-2) state) instrs
+        compile infos (pc + 1) (State.env_acc (-clo_offset_3) state) instrs
     | PUSHOFFSETCLOSURE0 ->
         let state = State.push state in
         compile infos (pc + 1) (State.env_acc 0 state) instrs
-    | PUSHOFFSETCLOSURE2 ->
+    | PUSHOFFSETCLOSURE3 ->
         let state = State.push state in
-        compile infos (pc + 1) (State.env_acc 2 state) instrs
+        compile infos (pc + 1) (State.env_acc clo_offset_3 state) instrs
     | PUSHOFFSETCLOSURE ->
         let state = State.push state in
         let n = gets code (pc + 1) in
@@ -2118,47 +2121,99 @@ let parse_bytecode code globals debug_data =
 (* HACK - override module *)
 
 let override_global =
-  let jsmodule name func =
-    Prim (Extern "%overrideMod", [ Pc (String name); Pc (String func) ])
-  in
-  [ ( "CamlinternalMod"
-    , fun _orig instrs ->
-        let x = Var.fresh_n "internalMod" in
-        let init_mod = Var.fresh_n "init_mod" in
-        let update_mod = Var.fresh_n "update_mod" in
-        ( x
-        , Let (x, Block (0, [| init_mod; update_mod |], NotArray))
-          :: Let (init_mod, jsmodule "CamlinternalMod" "init_mod")
-          :: Let (update_mod, jsmodule "CamlinternalMod" "update_mod")
-          :: instrs ) )
-  ]
+  match Ocaml_version.v with
+  | `V4_13 -> []
+  | `V4_04 | `V4_06 | `V4_07 | `V4_08 | `V4_09 | `V4_10 | `V4_11 | `V4_12 ->
+      let jsmodule name func =
+        Prim (Extern "%overrideMod", [ Pc (String name); Pc (String func) ])
+      in
+      [ ( "CamlinternalMod"
+        , fun _orig instrs ->
+            let x = Var.fresh_n "internalMod" in
+            let init_mod = Var.fresh_n "init_mod" in
+            let update_mod = Var.fresh_n "update_mod" in
+            ( x
+            , Let (x, Block (0, [| init_mod; update_mod |], NotArray))
+              :: Let (init_mod, jsmodule "CamlinternalMod" "init_mod")
+              :: Let (update_mod, jsmodule "CamlinternalMod" "update_mod")
+              :: instrs ) )
+      ]
 
 (* HACK END *)
 
-let seek_section toc ic name =
-  let rec seek_sec curr_ofs = function
-    | [] -> raise Not_found
-    | (n, len) :: rem ->
-        if String.equal n name
-        then (
-          seek_in ic (curr_ofs - len);
-          len)
-        else seek_sec (curr_ofs - len) rem
-  in
-  seek_sec (in_channel_length ic - 16 - (8 * List.length toc)) toc
+module Toc : sig
+  type t
 
-let read_toc ic =
-  let pos_trailer = in_channel_length ic - 16 in
-  seek_in ic pos_trailer;
-  let num_sections = input_binary_int ic in
-  seek_in ic (pos_trailer - (8 * num_sections));
-  let section_table = ref [] in
-  for _i = 1 to num_sections do
-    let name = really_input_string ic 4 in
-    let len = input_binary_int ic in
-    section_table := (name, len) :: !section_table
-  done;
-  !section_table
+  val read : in_channel -> t
+
+  val seek_section : t -> in_channel -> string -> int
+
+  val read_code : t -> in_channel -> string
+
+  val read_data : t -> in_channel -> Obj.t array
+
+  val read_crcs : t -> in_channel -> (string * Digest.t option) list
+
+  val read_prim : t -> in_channel -> string
+
+  val read_symb : t -> in_channel -> Ocaml_compiler.Symtable.GlobalMap.t
+end = struct
+  type t = (string * int) list
+
+  let seek_section toc ic name =
+    let rec seek_sec curr_ofs = function
+      | [] -> raise Not_found
+      | (n, len) :: rem ->
+          if String.equal n name
+          then (
+            seek_in ic (curr_ofs - len);
+            len)
+          else seek_sec (curr_ofs - len) rem
+    in
+    seek_sec (in_channel_length ic - 16 - (8 * List.length toc)) toc
+
+  let read ic =
+    let pos_trailer = in_channel_length ic - 16 in
+    seek_in ic pos_trailer;
+    let num_sections = input_binary_int ic in
+    seek_in ic (pos_trailer - (8 * num_sections));
+    let section_table = ref [] in
+    for _i = 1 to num_sections do
+      let name = really_input_string ic 4 in
+      let len = input_binary_int ic in
+      section_table := (name, len) :: !section_table
+    done;
+    !section_table
+
+  let read_code toc ic =
+    let code_size = seek_section toc ic "CODE" in
+    really_input_string ic code_size
+
+  let read_data toc ic =
+    ignore (seek_section toc ic "DATA");
+    let init_data : Obj.t array = input_value ic in
+    init_data
+
+  let read_symb toc ic =
+    ignore (seek_section toc ic "SYMB");
+    let orig_symbols : Ocaml_compiler.Symtable.GlobalMap.t = input_value ic in
+    orig_symbols
+
+  let read_crcs toc ic =
+    ignore (seek_section toc ic "CRCS");
+    let orig_crcs : (string * Digest.t option) list = input_value ic in
+    orig_crcs
+
+  let read_prim toc ic =
+    let prim_size = seek_section toc ic "PRIM" in
+    let prim = really_input_string ic prim_size in
+    prim
+end
+
+let read_primitives toc ic =
+  let prim = Toc.read_prim toc ic in
+  assert (Char.equal (String.get prim (String.length prim - 1)) '\000');
+  String.split_char ~sep:'\000' (String.sub prim ~pos:0 ~len:(String.length prim - 1))
 
 let from_exe
     ?(includes = [])
@@ -2168,19 +2223,14 @@ let from_exe
     ?(debug = false)
     ic =
   let debug_data = Debug.create ~toplevel debug in
-  let toc = read_toc ic in
-  let prim_size = seek_section toc ic "PRIM" in
-  let prim = really_input_string ic prim_size in
-  let primitive_table = Array.of_list (String.split_char ~sep:'\000' prim) in
-  let code_size = seek_section toc ic "CODE" in
-  let code = really_input_string ic code_size in
-  ignore (seek_section toc ic "DATA");
-  let init_data : Obj.t array = input_value ic in
+  let toc = Toc.read ic in
+  let primitives = read_primitives toc ic in
+  let primitive_table = Array.of_list primitives in
+  let code = Toc.read_code toc ic in
+  let init_data = Toc.read_data toc ic in
   let init_data = Array.map ~f:Constants.parse init_data in
-  ignore (seek_section toc ic "SYMB");
-  let orig_symbols : Ocaml_compiler.Symtable.GlobalMap.t = input_value ic in
-  ignore (seek_section toc ic "CRCS");
-  let orig_crcs : (string * Digest.t option) list = input_value ic in
+  let orig_symbols = Toc.read_symb toc ic in
+  let orig_crcs = Toc.read_crcs toc ic in
   let keeps =
     let t = Hashtbl.create 17 in
     List.iter ~f:(fun (_, s) -> Hashtbl.add t s ()) predefined_exceptions;
@@ -2206,7 +2256,7 @@ let from_exe
   then ()
   else
     try
-      ignore (seek_section toc ic "DBUG");
+      ignore (Toc.seek_section toc ic "DBUG");
       Debug.read debug_data ~crcs ~includes ic
     with Not_found ->
       if Debug.enabled debug_data || Debug.toplevel debug_data
@@ -2261,7 +2311,10 @@ let from_exe
     then
       (* Include linking information *)
       let toc =
-        [ "SYMB", Obj.repr symbols; "CRCS", Obj.repr crcs; "PRIM", Obj.repr prim ]
+        [ "SYMB", Obj.repr symbols
+        ; "CRCS", Obj.repr crcs
+        ; "PRIM", Obj.repr (String.concat ~sep:"\000" primitives ^ "\000")
+        ]
       in
       let gdata = Var.fresh () in
       let infos =
@@ -2561,7 +2614,14 @@ let predefined_exceptions () =
         let v_index = Var.fresh () in
         [ Let (v_name, Constant (String name))
         ; Let (v_name_js, Constant (IString name))
-        ; Let (v_index, Constant (Int (Int32.of_int (-index))))
+        ; Let
+            ( v_index
+            , Constant
+                (Int
+                   ((* Predefined exceptions are registered in
+                       Symtable.init with [-index - 1] *)
+                    Int32.of_int
+                      (-index - 1))) )
         ; Let (exn, Block (248, [| v_name; v_index |], NotArray))
         ; Let
             ( Var.fresh ()
