@@ -96,6 +96,11 @@ module List = struct
 
   let slow_map l ~f = rev (rev_map ~f l)
 
+  let max_non_tailcall =
+    match Sys.backend_type with
+    | Sys.Native | Sys.Bytecode -> 1_000
+    | Sys.Other _ -> 50
+
   let rec count_map ~f l ctr =
     match l with
     | [] -> []
@@ -128,7 +133,7 @@ module List = struct
         :: f3
         :: f4
         :: f5
-        :: (if ctr > 1000 then slow_map ~f tl else count_map ~f tl (ctr + 1))
+        :: (if ctr > max_non_tailcall then slow_map ~f tl else count_map ~f tl (ctr + 1))
 
   let map l ~f = count_map ~f l 0
 
@@ -175,8 +180,59 @@ module List = struct
           | `Snd y -> loop t fst (y :: snd))
     in
     loop t [] []
+
+  let tail_append l1 l2 = rev_append (rev l1) l2
+
+  let rec count_append l1 l2 count =
+    match l2 with
+    | [] -> l1
+    | _ -> (
+        match l1 with
+        | [] -> l2
+        | [ x1 ] -> x1 :: l2
+        | [ x1; x2 ] -> x1 :: x2 :: l2
+        | [ x1; x2; x3 ] -> x1 :: x2 :: x3 :: l2
+        | [ x1; x2; x3; x4 ] -> x1 :: x2 :: x3 :: x4 :: l2
+        | x1 :: x2 :: x3 :: x4 :: x5 :: tl ->
+            x1
+            :: x2
+            :: x3
+            :: x4
+            :: x5
+            ::
+            (if count > max_non_tailcall
+            then tail_append tl l2
+            else count_append tl l2 (count + 1)))
+
+  let append l1 l2 = count_append l1 l2 0
+
+  let group l ~f =
+    let rec loop (l : 'a list) (this_group : 'a list) (acc : 'a list list) : 'a list list
+        =
+      match l with
+      | [] -> List.rev (List.rev this_group :: acc)
+      | x :: xs ->
+          let pred = List.hd this_group in
+          if f x pred
+          then loop xs (x :: this_group) acc
+          else loop xs [ x ] (List.rev this_group :: acc)
+    in
+    match l with
+    | [] -> []
+    | x :: xs -> loop xs [ x ] []
+
+  let concat_map ~f l =
+    let rec aux f acc = function
+      | [] -> rev acc
+      | x :: l ->
+          let xs = f x in
+          aux f (rev_append xs acc) l
+    in
+    aux f [] l
 end
 
+let ( @ ) = List.append
+
 module Nativeint = struct
   include Nativeint
 
@@ -242,6 +298,11 @@ module Option = struct
     | None -> None
     | Some v -> Some (f v)
 
+  let bind ~f x =
+    match x with
+    | None -> None
+    | Some v -> f v
+
   let iter ~f x =
     match x with
     | None -> ()
@@ -425,16 +486,7 @@ module String = struct
     done;
     !res
 
-  let split_char ~sep p =
-    let len = String.length p in
-    let rec split beg cur =
-      if cur >= len
-      then if cur - beg > 0 then [ String.sub p beg (cur - beg) ] else []
-      else if Char.equal p.[cur] sep
-      then String.sub p beg (cur - beg) :: split (cur + 1) (cur + 1)
-      else split beg (cur + 1)
-    in
-    split 0 0
+  let split_char ~sep p = String.split_on_char sep p
 
   (* copied from https://github.com/ocaml/ocaml/pull/10 *)
   let split ~sep s =
