@@ -293,6 +293,10 @@ and mouseEvent =
 
     method screenY : int readonly_prop
 
+    method offsetX : int readonly_prop
+
+    method offsetY : int readonly_prop
+
     method ctrlKey : bool t readonly_prop
 
     method shiftKey : bool t readonly_prop
@@ -414,6 +418,13 @@ and touch =
     method pageY : int readonly_prop
   end
 
+and submitEvent =
+  object
+    inherit event
+
+    method submitter : element t optdef readonly_prop
+  end
+
 and dragEvent =
   object
     inherit mouseEvent
@@ -421,6 +432,13 @@ and dragEvent =
     method dataTransfer : dataTransfer t readonly_prop
   end
 
+and clipboardEvent =
+  object
+    inherit event
+
+    method clipboardData : dataTransfer t readonly_prop
+  end
+
 and dataTransfer =
   object
     method dropEffect : js_string t prop
@@ -653,6 +671,8 @@ and element =
 
     method textContent : js_string t opt prop
 
+    method innerText : js_string t prop
+
     method clientLeft : int readonly_prop
 
     method clientTop : int readonly_prop
@@ -729,6 +749,12 @@ module Event = struct
 
   let click = Dom.Event.make "click"
 
+  let copy = Dom.Event.make "copy"
+
+  let cut = Dom.Event.make "cut"
+
+  let paste = Dom.Event.make "paste"
+
   let dblclick = Dom.Event.make "dblclick"
 
   let mousedown = Dom.Event.make "mousedown"
@@ -1011,7 +1037,7 @@ class type formElement =
 
     method reset : unit meth
 
-    method onsubmit : ('self t, event t) event_listener writeonly_prop
+    method onsubmit : ('self t, submitEvent t) event_listener writeonly_prop
   end
 
 class type optGroupElement =
@@ -2064,6 +2090,12 @@ class type document =
 
     method activeElement : element t opt readonly_prop
 
+    method hidden : bool t readonly_prop
+
+    method onfullscreenchange : (document t, event t) event_listener writeonly_prop
+
+    method onwebkitfullscreenchange : (document t, event t) event_listener writeonly_prop
+
     inherit eventTarget
   end
 
