@@ -289,16 +289,20 @@ and mouseEvent =
 
     method relatedTarget : element t opt optdef readonly_prop
 
+    (* Relative to viewport *)
     method clientX : int readonly_prop
 
-    (* Relative to viewport *)
     method clientY : int readonly_prop
 
+    (* Relative to the edge of the screen *)
     method screenX : int readonly_prop
 
-    (* Relative to the edge of the screen *)
     method screenY : int readonly_prop
 
+    method offsetX : int readonly_prop
+
+    method offsetY : int readonly_prop
+
     method ctrlKey : bool t readonly_prop
 
     method shiftKey : bool t readonly_prop
@@ -423,6 +427,13 @@ and touch =
     method pageY : int readonly_prop
   end
 
+and submitEvent =
+  object
+    inherit event
+
+    method submitter : element t optdef readonly_prop
+  end
+
 and dragEvent =
   object
     inherit mouseEvent
@@ -430,6 +441,13 @@ and dragEvent =
     method dataTransfer : dataTransfer t readonly_prop
   end
 
+and clipboardEvent =
+  object
+    inherit event
+
+    method clipboardData : dataTransfer t readonly_prop
+  end
+
 and dataTransfer =
   object
     method dropEffect : js_string t prop
@@ -668,6 +686,8 @@ and element =
 
     method textContent : js_string t opt prop
 
+    method innerText : js_string t prop
+
     method clientLeft : int readonly_prop
 
     method clientTop : int readonly_prop
@@ -841,7 +861,7 @@ class type formElement =
 
     method reset : unit meth
 
-    method onsubmit : ('self t, event t) event_listener writeonly_prop
+    method onsubmit : ('self t, submitEvent t) event_listener writeonly_prop
   end
 
 class type optGroupElement =
@@ -1912,6 +1932,12 @@ class type document =
 
     method activeElement : element t opt readonly_prop
 
+    method hidden : bool t readonly_prop
+
+    method onfullscreenchange : (document t, event t) event_listener writeonly_prop
+
+    method onwebkitfullscreenchange : (document t, event t) event_listener writeonly_prop
+
     inherit eventTarget
   end
 
@@ -2281,6 +2307,12 @@ module Event : sig
 
   val click : mouseEvent t typ
 
+  val copy : clipboardEvent t typ
+
+  val cut : clipboardEvent t typ
+
+  val paste : clipboardEvent t typ
+
   val dblclick : mouseEvent t typ
 
   val mousedown : mouseEvent t typ
@@ -2333,7 +2365,7 @@ module Event : sig
 
   val timeupdate : event t typ
 
-  val submit : event t typ
+  val submit : submitEvent t typ
 
   val scroll : event t typ
 
